/* $ Id: $
 * User: kortenka
 * Date: 29.10.2002
 * Time: 23:13:37
 */
package eaa.sorting.helper;

import java.io.File;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

public class ClassEnumerator {
    private static final String pathSep = System.getProperty("path.separator");
    private static final String fileSep = System.getProperty("file.separator");

    static public Enumeration getAllClassesImplementing(String interfaceName) {
        Enumeration allClasses = getAllClasses();
        Vector foundClasses = new Vector();

        while (allClasses.hasMoreElements()) {
            String classname = className((String) allClasses.nextElement());
            Class c;
            try {
                c = Class.forName(classname);
                Class[] interfaces = c.getInterfaces();
                for (int i = 0; i < interfaces.length; i++) {
                    Class anInterface = interfaces[i];
                    if (anInterface.getName().equals(interfaceName)) {
                        foundClasses.add(classname);
                    }

                }
            } catch (ClassNotFoundException e) {
            }
        }
        return foundClasses.elements();
    }

    private static String className(String s) {
        StringTokenizer st = new StringTokenizer(s, fileSep);
        StringBuffer sb = new StringBuffer(s.length());
        boolean first = true;
        while (st.hasMoreElements()) {
            String string = (String) st.nextElement();
            if (string.endsWith(".class")) {
                if (!first)
                    sb.append(".");
                sb.append(string.substring(0, string.indexOf('.')));
            } else {
                if (!first)
                    sb.append(".");
                sb.append(string);
            }
            first = false;
        }
        return sb.toString();
    }

    static public Enumeration getAllClasses() {
        Hashtable resources = new Hashtable();
        Vector classes = new Vector();

        String classpath = System.getProperty("java.class.path");

        Enumeration pathEntries = enumerateEntries(classpath, pathSep);

        while (pathEntries.hasMoreElements()) {
            String rootElement = (String) pathEntries.nextElement();
            enumerateFiles(resources, "", rootElement);
        }

        Enumeration en = resources.keys();
        while (en.hasMoreElements()) {
            String s = (String) en.nextElement();
            if (resources.get(s).equals("class")) {
                classes.add(s);
            }
        }

        return classes.elements();
    }

    private static void enumerateFiles(Hashtable classes, String filename, String rootElement) {
        if (classes.contains(filename))
            return;
        File f = new File(rootElement, filename);
        if (f.isDirectory()) {
            classes.put(filename, "dir");
            String[] files = f.list();
            for (int i = 0; i < files.length; i++) {
                String file = files[i];
                enumerateFiles(classes, filename.concat(fileSep).concat(file), rootElement);
            }
        } else if (f.isFile()) {
            if (filename.endsWith(".class")) {
                classes.put(filename, "class");
            } else {
                classes.put(filename, "file");
            }
        }
    }

    private static Enumeration enumerateEntries(String classpath, String pathSep) {
        StringTokenizer st = new StringTokenizer(classpath, pathSep);
        return st;
    }
}
